import os

import PyIPSDK
import PyIPSDK.IPSDKIPLRegistration as registration
import PyIPSDK.IPSDKIPLUtility as util
import PyIPSDK.IPSDKIPLGeometricTransform as gtrans
import PyIPSDK.IPSDKIPLGlobalMeasure as gblmsr
import PyIPSDK.IPSDKIPLArithmetic as arithm
import PyIPSDK.IPSDKIPLFiltering as filter

import numpy as np

def registration3d(inImage,angle):

	nbPointsRegistration = 200
	
	def getTransform(greySignaturesFixed, greySignaturesMoving, correlationThreshold, estimationConfig):

		# Calcul du recalage  appliquer en utilisant les descripteurs de chaque image

		theta = 0
		thetaBin = 0.05
		maxTx =50
		maxTy =50

		outRegistrationResult = registration.greySignedFeatures2dRegistration(greySignaturesMoving, greySignaturesFixed, PyIPSDK.eRegistrationMotionModel2d.eRMM2d_Rigid, correlationThreshold, estimationConfig)

		dictPt = {}
		i = 0

		for ind in outRegistrationResult.indicators.assignments:
			dictPt[i] = {}
			dictPt[i]["x1"] = greySignaturesFixed.signatureColl[ind.ptIdx2].x
			dictPt[i]["y1"] = greySignaturesFixed.signatureColl[ind.ptIdx2].y
			dictPt[i]["x2"] = greySignaturesMoving.signatureColl[ind.ptIdx1].x
			dictPt[i]["y2"] = greySignaturesMoving.signatureColl[ind.ptIdx1].y
			dictPt[i]["corr"] = 0

			i += 1

		# print ("number of assignements : " + str(i))

		for i in dictPt:
			for j in dictPt:
				if j != i:
					v1x, v1y = dictPt[i]["x1"] - dictPt[j]["x1"], dictPt[i]["y1"] - dictPt[j]["y1"]
					v2x, v2y = dictPt[i]["x2"] - dictPt[j]["x2"], dictPt[i]["y2"] - dictPt[j]["y2"]

					value = (v1x * v2x + v1y * v2y) / (np.sqrt(v1x * v1x + v1y * v1y) * np.sqrt(v2x * v2x + v2y * v2y))
					value = max(-1, value)
					value = min(value, 1)

					signe = v2x * v1y - v2y * v1x
					if signe < 0:
						signe = -1
					else:
						signe = 1

					thetaValue = (np.arccos(value) * signe)
					if thetaValue >= theta - thetaBin and thetaValue <= theta + thetaBin:
						dictPt[i]["corr"] += 1

		nbPoints = 0
		Tx = 0
		Ty = 0
		for i in dictPt:
			if dictPt[i]["corr"] >= 3 and abs(dictPt[i]["x2"] - dictPt[i]["x1"]) <= maxTx and abs(dictPt[i]["y2"] - dictPt[i]["y1"]) <= maxTy:
				Tx += (dictPt[i]["x2"] - dictPt[i]["x1"]) * dictPt[i]["corr"]
				Ty += (dictPt[i]["y2"] - dictPt[i]["y1"]) * dictPt[i]["corr"]
				nbPoints += dictPt[i]["corr"]
		# print ("nbPoints : "+str(nbPoints))

		if nbPoints != 0:
			Tx /= nbPoints
			Ty /= nbPoints

		print("Tx : " + str(Tx) + " , Ty : " + str(Ty))

		return -Tx, -Ty, nbPoints

	dictImages = {}

	displacement = np.tan(angle*np.pi/180)

	# Recalage des coupes 2d d'un dossier deux par deux

	correlationThreshold = 0.95
	estimationConfig = PyIPSDK.EstimationConfig()
	estimationConfig.initLMS(0.48)

	sizeZ = inImage.getSizeZ()

	dx = 0
	dy = 0

	minDx = 0
	maxDx = 0
	minDy = 0
	maxDy = 0

	for i in range(sizeZ-1):

		print (i)

		if i == 0:
			imageFixed = PyIPSDK.extractPlan(i, 0, 0, inImage)
			imageFixed = util.copyImg(imageFixed)
			greySignaturesFixed = registration.extractGreySignedFeatures2d(imageFixed, nbPointsRegistration)
			dictImages[0] = {}
			dictImages[0]["Tx"] = 0
			dictImages[0]["Ty"] = 0
		else:
			greySignaturesFixed = greySignaturesMoving

		imageMoving = PyIPSDK.extractPlan(i+1, 0, 0, inImage)

		greySignaturesMoving = registration.extractGreySignedFeatures2d(imageMoving, nbPointsRegistration)

		Tx, Ty, nbPoints = getTransform(greySignaturesFixed, greySignaturesMoving, correlationThreshold, estimationConfig)

		dictImages[i + 1] = {}
		dictImages[i + 1]["Tx"] = Tx
		dictImages[i + 1]["Ty"] = Ty

	for i in range(len(dictImages)):

		dictImages[i]["Tx"] = dictImages[i]["Tx"]
		dictImages[i]["Ty"] = dictImages[i]["Ty"] + displacement

		dx += dictImages[i]["Tx"]
		dy += dictImages[i]["Ty"]

		minDx = min(dx,minDx)
		maxDx = max(dx,maxDx)
		minDy = min(dy,minDy)
		maxDy = max(dy,maxDy)

	minDx = int(abs(minDx))
	maxDx = int(abs(maxDx))
	minDy = int(abs(minDy))
	maxDy = int(abs(maxDy))

	dx = 0
	dy = 0

	outImage = PyIPSDK.createImage(inImage.getBufferType(),inImage.getSizeX() + maxDx - minDx,inImage.getSizeY() + maxDy - minDy,sizeZ)
	util.eraseImg(outImage, 0)

	for i in range(len(dictImages)):

		print ("Image : "+str(i))

		imageFixed = PyIPSDK.extractPlan(i, 0, 0, inImage)
		imageFixed = util.copyImg(imageFixed)

		dx = dx + dictImages[i]["Tx"]
		dy = dy + dictImages[i]["Ty"]

		transform1 = PyIPSDK.createWarpMotionTransform2d(PyIPSDK.eGeometricTransform2dType.eGT2DT_Translation, [int(dx - minDx), int(dy - minDy)])
		outPlan = PyIPSDK.extractPlan(i,0,0,outImage)
		gtrans.warp2dImg(imageFixed, transform1, outPlan)

	return outImage









